==> LockingWorkflow-0_1.tgz: LockingWorkflow/LockingWorkflow.py <==
##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

""" creates the two-state locking workflow.
"""

from AccessControl.Permissions import view
from Products.ExternalMethod.ExternalMethod import \
     manage_addExternalMethod as addScript

from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition

p_view = view
p_modify = ModifyPortalContent

r_manager = 'Manager'
r_owner = 'Owner'
r_editor = 'Editor'


def setupLockingWorkflow(wf):
    """
    Sets up a DCWorkflow with two states 'unlocked' and 'locked'
    and transitions 'lock' and 'unlock' between them 
    """
    wf.setProperties(title='Looking Workflow')

    for s in ('unlocked', 'locked'):
        wf.states.addState(s)
    for t in ('lock', 'unlock'):
        wf.transitions.addTransition(t)
    for v in ('action', 'actor', 'comments', 'locking_history', 'time'):
        wf.variables.addVariable(v)
    for p in (p_modify,):
        wf.addManagedPermission(p)

    for s in ('setEditorRole', 'unsetEditorRole'):
        addScript(wf.scripts,
                  id=s,
                  title="from the locking workflow",
                  module='LockingWorkflow.WorkflowScripts',
                  function=s)

    wf.states.setInitialState('unlocked')

    sdef = wf.states['unlocked']
    sdef.setProperties(
        title='Everyone with edit rights can start working on the item',
        transitions=('lock',))
    sdef.setPermission(p_modify, 1, ())

    sdef = wf.states['locked']
    sdef.setProperties(
        title='Only the current editor can work on the item',
        transitions=('unlock',))
    sdef.setPermission(p_modify, 0, (r_editor,))


    tdef = wf.transitions['lock']
    tdef.setProperties(
        title='User becomes exclusive editor',
        new_state_id='locked',
	script_name='setEditorRole',
        ## actbox_name='lock',  # XXX for debugging only; remove when done
        actbox_url='',
        props={'guard_permission':p_modify})

    tdef = wf.transitions['unlock']
    tdef.setProperties(
        title='Remove lock and delete editor local role',
        new_state_id='unlocked',
	after_script_name='unsetEditorRole',
        actbox_name='unlock',
        actbox_url='',
        props={'guard_roles':r_manager})


    wf.variables.setStateVar('locking_state')


    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getId',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['locking_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       )

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)


def createLockingWorkflow(id):
    '''
    '''
    ob = DCWorkflowDefinition(id)
    setupLockingWorkflow(ob)
    return ob

addWorkflowFactory(createLockingWorkflow, id='locking_workflow',
                   title='Two-state locking workflow')

==> LockingWorkflow-0_2_0.tgz: LockingWorkflow/LockingWorkflow.py <==
##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

""" creates the two-state locking workflow.
"""

from AccessControl.Permissions import view
from Products.ExternalMethod.ExternalMethod import \
     manage_addExternalMethod as addScript

from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition

p_view = view
p_modify = ModifyPortalContent

r_manager = 'Manager'
r_owner = 'Owner'
r_editor = 'Editor'


def setupLockingWorkflow(wf):
    """
    Sets up a DCWorkflow with two states 'unlocked' and 'locked'
    and transitions 'lock' and 'unlock' between them 
    """
    wf.setProperties(title='Looking Workflow')

    for s in ('unlocked', 'locked'):
        wf.states.addState(s)
    for t in ('lock', 'unlock'):
        wf.transitions.addTransition(t)
    for v in ('action', 'actor', 'comments', 'locking_history', 'time'):
        wf.variables.addVariable(v)
    for p in (p_modify,):
        wf.addManagedPermission(p)

    for s in ('setEditorRole', 'unsetEditorRole'):
        addScript(wf.scripts,
                  id=s,
                  title="from the locking workflow",
                  module='LockingWorkflow.WorkflowScripts',
                  function=s)

    wf.states.setInitialState('unlocked')

    sdef = wf.states['unlocked']
    sdef.setProperties(
        title='Everyone with edit rights can start working on the item',
        transitions=('lock',))
    sdef.setPermission(p_modify, 1, ())

    sdef = wf.states['locked']
    sdef.setProperties(
        title='Only the current editor can work on the item',
        transitions=('unlock',))
    sdef.setPermission(p_modify, 0, (r_editor,))


    tdef = wf.transitions['lock']
    tdef.setProperties(
        title='User becomes exclusive editor',
        new_state_id='locked',
	script_name='setEditorRole',
        ## actbox_name='lock',  # XXX for debugging only; remove when done
        actbox_url='',
        props={'guard_permission':p_modify})

    tdef = wf.transitions['unlock']
    tdef.setProperties(
        title='Remove lock and delete editor local role',
        new_state_id='unlocked',
	after_script_name='unsetEditorRole',
        actbox_name='unlock',
        actbox_url='',
        props={'guard_roles':';'.join((r_manager, r_editor))})


    wf.variables.setStateVar('locking_state')


    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getId',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['locking_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       )

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)


def createLockingWorkflow(id):
    '''
    '''
    ob = DCWorkflowDefinition(id)
    setupLockingWorkflow(ob)
    return ob

addWorkflowFactory(createLockingWorkflow, id='locking_workflow',
                   title='Two-state locking workflow')
